@@ -29,17 +29,17 @@ func NewConfig() *Config {
 // NOTE: If you any new setting here, please also add a value for it at testdata/rull-reference.toml
 type Config struct {
 	AppName       string                    `toml:"app,omitempty" json:"app,omitempty"`
-	KillSignal    string                    `toml:"kill_signal,omitempty" json:"kill_signal,omitempty"`
-	KillTimeout   int                       `toml:"kill_timeout,omitempty" json:"kill_timeout,omitempty"`
+	KillSignal    *string                   `toml:"kill_signal,omitempty" json:"kill_signal,omitempty"`
+	KillTimeout   *int                      `toml:"kill_timeout,omitempty" json:"kill_timeout,omitempty"`
 	PrimaryRegion string                    `toml:"primary_region,omitempty" json:"primary_region,omitempty"`
 	Experimental  *Experimental             `toml:"experimental,omitempty" json:"experimental,omitempty"`
 	Build         *Build                    `toml:"build,omitempty" json:"build,omitempty"`
 	Deploy        *Deploy                   `toml:"deploy, omitempty" json:"deploy,omitempty"`
 	Env           map[string]string         `toml:"env,omitempty" json:"env,omitempty"`
-	HttpService   *HTTPService              `toml:"http_service,omitempty" json:"http_service,omitempty"`
+	HTTPService   *HTTPService              `toml:"http_service,omitempty" json:"http_service,omitempty"`
 	Metrics       *api.MachineMetrics       `toml:"metrics,omitempty" json:"metrics,omitempty"`
 	Statics       []Static                  `toml:"statics,omitempty" json:"statics,omitempty"`
-	Mounts        *Volume                   `toml:"mounts,omitempty" json:"mounts,omitempty"`
+	Mounts        []Mount                   `toml:"mounts,omitempty" json:"mounts,omitempty"`
 	Processes     map[string]string         `toml:"processes,omitempty" json:"processes,omitempty"`
 	Checks        map[string]*ToplevelCheck `toml:"checks,omitempty" json:"checks,omitempty"`
 	Services      []Service                 `toml:"services,omitempty" json:"services,omitempty"`
@@ -72,9 +72,10 @@ type Static struct {
 	UrlPrefix string `toml:"url_prefix" json:"url_prefix,omitempty" validate:"required"`
 }
 
-type Volume struct {
-	Source      string `toml:"source,omitempty" json:"source,omitempty"`
-	Destination string `toml:"destination" json:"destination,omitempty"`
+type Mount struct {
+	Source      string   `toml:"source,omitempty" json:"source,omitempty"`
+	Destination string   `toml:"destination" json:"destination,omitempty"`
+	Processes   []string `json:"processes,omitempty" toml:"processes,omitempty"`
 }
 
 type Build struct {
@@ -152,16 +153,9 @@ func (c *Config) DockerBuildTarget() string {
 	return c.Build.DockerBuildTarget
 }
 
-func (c *Config) MountsDestination() string {
-	if c == nil || c.Mounts == nil {
-		return ""
-	}
-	return c.Mounts.Destination
-}
-
 func (c *Config) InternalPort() int {
-	if c.HttpService != nil {
-		return c.HttpService.InternalPort
+	if c.HTTPService != nil {
+		return c.HTTPService.InternalPort
 	}
 
 	if len(c.Services) > 0 {
@@ -196,3 +190,12 @@ func (cfg *Config) BuildStrategies() []string {
 
 	return strategies
 }
+
+func (c *Config) Volumes() []Mount {
+	if mounts, ok := c.RawDefinition["mounts"]; ok {
+		if arr, ok := mounts.([]Mount); ok {
+			return arr
+		}
+	}
+	return c.Mounts
+}
