@@ -145,10 +145,10 @@ func TestLoadTOMLAppConfigMountsArray(t *testing.T) {
 		configFilePath:   "./testdata/mounts-array.toml",
 		defaultGroupName: "app",
 		AppName:          "foo",
-		Mounts: &Volume{
+		Mounts: []Mount{{
 			Source:      "pg_data",
 			Destination: "/data",
-		},
+		}},
 		RawDefinition: map[string]any{
 			"app": "foo",
 			"mounts": []map[string]any{{
@@ -171,10 +171,10 @@ func TestLoadTOMLAppConfigOldFormat(t *testing.T) {
 			"FOO": "STRING",
 			"BAR": "123",
 		},
-		Mounts: &Volume{
+		Mounts: []Mount{{
 			Source:      "data",
 			Destination: "/data",
-		},
+		}},
 		Services: []Service{
 			{
 				InternalPort: 8080,
@@ -191,22 +191,22 @@ func TestLoadTOMLAppConfigOldFormat(t *testing.T) {
 				},
 				TCPChecks: []*ServiceTCPCheck{
 					{
-						Interval: mustParseDuration("10s"),
-						Timeout:  mustParseDuration("2s"),
+						Interval: api.MustParseDuration("10s"),
+						Timeout:  api.MustParseDuration("2s"),
 					},
 					{
-						Interval: mustParseDuration("20s"),
-						Timeout:  mustParseDuration("3s"),
+						Interval: api.MustParseDuration("20s"),
+						Timeout:  api.MustParseDuration("3s"),
 					},
 				},
 				HTTPChecks: []*ServiceHTTPCheck{
 					{
-						Interval: mustParseDuration("30s"),
-						Timeout:  mustParseDuration("4s"),
+						Interval: api.MustParseDuration("30s"),
+						Timeout:  api.MustParseDuration("4s"),
 					},
 					{
-						Interval: mustParseDuration("20s"),
-						Timeout:  mustParseDuration("3s"),
+						Interval: api.MustParseDuration("20s"),
+						Timeout:  api.MustParseDuration("3s"),
 					},
 				},
 			},
@@ -283,8 +283,8 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 		configFilePath:   "./testdata/full-reference.toml",
 		defaultGroupName: "app",
 		AppName:          "foo",
-		KillSignal:       "SIGTERM",
-		KillTimeout:      3,
+		KillSignal:       api.Pointer("SIGTERM"),
+		KillTimeout:      api.Pointer(3),
 		PrimaryRegion:    "sea",
 		Experimental: &Experimental{
 			Cmd:          []string{"cmd"},
@@ -328,9 +328,9 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 			Path: "/metrics",
 		},
 
-		HttpService: &HTTPService{
+		HTTPService: &HTTPService{
 			InternalPort: 8080,
-			ForceHttps:   true,
+			ForceHTTPS:   true,
 			Concurrency: &api.MachineServiceConcurrency{
 				Type:      "donuts",
 				HardLimit: 10,
@@ -345,10 +345,10 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 			},
 		},
 
-		Mounts: &Volume{
+		Mounts: []Mount{{
 			Source:      "data",
 			Destination: "/data",
-		},
+		}},
 
 		Processes: map[string]string{
 			"web":  "run web",
@@ -359,9 +359,9 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 			"status": {
 				Port:              api.Pointer(2020),
 				Type:              api.Pointer("http"),
-				Interval:          mustParseDuration("10s"),
-				Timeout:           mustParseDuration("2s"),
-				GracePeriod:       mustParseDuration("27s"),
+				Interval:          api.MustParseDuration("10s"),
+				Timeout:           api.MustParseDuration("2s"),
+				GracePeriod:       api.MustParseDuration("27s"),
 				HTTPMethod:        api.Pointer("GET"),
 				HTTPPath:          api.Pointer("/status"),
 				HTTPProtocol:      api.Pointer("https"),
@@ -397,18 +397,18 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 
 				TCPChecks: []*ServiceTCPCheck{
 					{
-						Interval:     mustParseDuration("21s"),
-						Timeout:      mustParseDuration("4s"),
-						GracePeriod:  mustParseDuration("1s"),
+						Interval:     api.MustParseDuration("21s"),
+						Timeout:      api.MustParseDuration("4s"),
+						GracePeriod:  api.MustParseDuration("1s"),
 						RestartLimit: 3,
 					},
 				},
 
 				HTTPChecks: []*ServiceHTTPCheck{
 					{
-						Interval:          mustParseDuration("81s"),
-						Timeout:           mustParseDuration("7s"),
-						GracePeriod:       mustParseDuration("2s"),
+						Interval:          api.MustParseDuration("81s"),
+						Timeout:           api.MustParseDuration("7s"),
+						GracePeriod:       api.MustParseDuration("2s"),
 						RestartLimit:      4,
 						HTTPMethod:        api.Pointer("GET"),
 						HTTPPath:          api.Pointer("/"),
@@ -419,8 +419,8 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 						},
 					},
 					{
-						Interval:   mustParseDuration("33s"),
-						Timeout:    mustParseDuration("10s"),
+						Interval:   api.MustParseDuration("33s"),
+						Timeout:    api.MustParseDuration("10s"),
 						HTTPMethod: api.Pointer("POST"),
 						HTTPPath:   api.Pointer("/check2"),
 					},
@@ -429,11 +429,3 @@ func TestLoadTOMLAppConfigReferenceFormat(t *testing.T) {
 		},
 	}, cfg)
 }
-
-func mustParseDuration(v any) *api.Duration {
-	d := &api.Duration{}
-	if err := d.ParseDuration(v); err != nil {
-		panic(err)
-	}
-	return d
-}
