@@ -16,9 +16,7 @@ import (
 	"github.com/superfly/flyctl/gql"
 	"github.com/superfly/flyctl/internal/appconfig"
 	"github.com/superfly/flyctl/internal/cmdutil"
-	"github.com/superfly/flyctl/internal/logger"
 	"github.com/superfly/flyctl/internal/machine"
-	"github.com/superfly/flyctl/internal/prompt"
 	"github.com/superfly/flyctl/iostreams"
 	"github.com/superfly/flyctl/terminal"
 )
@@ -38,12 +36,10 @@ type MachineDeploymentArgs struct {
 	Strategy          string
 	EnvFromFlags      []string
 	PrimaryRegionFlag string
-	BuildOnly         bool
 	SkipHealthChecks  bool
 	RestartOnly       bool
 	WaitTimeout       time.Duration
 	LeaseTimeout      time.Duration
-	NewVolumeName     string
 }
 
 type machineDeployment struct {
@@ -57,7 +53,7 @@ type machineDeployment struct {
 	img                   string
 	machineSet            machine.MachineSet
 	releaseCommandMachine machine.MachineSet
-	volumes               []api.Volume
+	volumes               map[string][]api.Volume
 	strategy              string
 	releaseId             string
 	releaseVersion        int
@@ -66,6 +62,7 @@ type machineDeployment struct {
 	waitTimeout           time.Duration
 	leaseTimeout          time.Duration
 	leaseDelayBetween     time.Duration
+	isFirstDeploy         bool
 }
 
 func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (MachineDeployment, error) {
@@ -79,9 +76,6 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 	if err != nil {
 		return nil, err
 	}
-	if args.NewVolumeName != "" && appConfig.Mounts != nil {
-		appConfig.Mounts.Source = args.NewVolumeName
-	}
 	err, _ = appConfig.Validate(ctx)
 	if err != nil {
 		return nil, err
@@ -136,6 +130,12 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 	if err != nil {
 		return nil, err
 	}
+	if err := md.setFirstDeploy(ctx); err != nil {
+		return nil, err
+	}
+	if err := md.provisionFirstDeploy(ctx); err != nil {
+		return nil, err
+	}
 	err = md.setImg(ctx)
 	if err != nil {
 		return nil, err
@@ -148,10 +148,6 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 	if err != nil {
 		return nil, err
 	}
-	err = md.provisionIpsOnFirstDeploy(ctx)
-	if err != nil {
-		return nil, err
-	}
 	err = md.createReleaseInBackend(ctx)
 	if err != nil {
 		return nil, err
@@ -159,6 +155,11 @@ func NewMachineDeployment(ctx context.Context, args MachineDeploymentArgs) (Mach
 	return md, nil
 }
 
+func (md *machineDeployment) setFirstDeploy(ctx context.Context) error {
+	md.isFirstDeploy = !md.app.Deployed || md.machineSet.IsEmpty()
+	return nil
+}
+
 func (md *machineDeployment) setMachinesForDeployment(ctx context.Context) error {
 	machines, releaseCmdMachine, err := md.flapsClient.ListFlyAppsMachines(ctx)
 	if err != nil {
@@ -195,7 +196,7 @@ func (md *machineDeployment) setMachinesForDeployment(ctx context.Context) error
 }
 
 func (md *machineDeployment) setVolumeConfig(ctx context.Context) error {
-	if md.appConfig.Mounts == nil {
+	if len(md.appConfig.Mounts) == 0 {
 		return nil
 	}
 
@@ -204,36 +205,83 @@ func (md *machineDeployment) setVolumeConfig(ctx context.Context) error {
 		return fmt.Errorf("Error fetching application volumes: %w", err)
 	}
 
-	md.volumes = lo.Filter(volumes, func(v api.Volume, _ int) bool {
-		return v.Name == md.appConfig.Mounts.Source && v.AttachedAllocation == nil && v.AttachedMachine == nil
+	unattached := lo.Filter(volumes, func(v api.Volume, _ int) bool {
+		return v.AttachedAllocation == nil && v.AttachedMachine == nil
+	})
+
+	md.volumes = lo.GroupBy(unattached, func(v api.Volume) string {
+		return v.Name
 	})
 	return nil
 }
 
 func (md *machineDeployment) validateVolumeConfig() error {
-	volumeDestination := ""
-	if md.appConfig.Mounts != nil {
-		volumeDestination = md.appConfig.Mounts.Destination
-	}
+	machineGroups := lo.GroupBy(
+		lo.Map(md.machineSet.GetMachines(), func(lm machine.LeasableMachine, _ int) *api.Machine {
+			return lm.Machine()
+		}),
+		func(m *api.Machine) string {
+			return m.ProcessGroup()
+		})
 
-	for _, m := range md.machineSet.GetMachines() {
-		mid := m.Machine().ID
-		mountsConfig := m.Machine().Config.Mounts
-		if len(mountsConfig) > 1 {
-			return fmt.Errorf("error machine %s has %d mounts and expected 1", mid, len(mountsConfig))
+	for _, groupName := range md.appConfig.ProcessNames() {
+		groupConfig, err := md.appConfig.Flatten(groupName)
+		if err != nil {
+			return err
 		}
-		if volumeDestination == "" && len(mountsConfig) != 0 {
-			return fmt.Errorf("error machine %s has a volume mounted and app config does not specify a volume; remove the volume from the machine or add a [mounts] configuration to fly.toml", mid)
-		}
-		if volumeDestination != "" && len(mountsConfig) == 0 {
-			return fmt.Errorf("error machine %s does not have a volume configured and fly.toml expects one with destination %s; remove the [mounts] configuration in fly.toml or use the machines API to add a volume to this machine", mid, volumeDestination)
+
+		switch ms := machineGroups[groupName]; len(ms) > 0 {
+		case true:
+			// For groups with machines, check the attached volumes match expected mounts
+			var mntSrc, mntDst string
+			if len(groupConfig.Mounts) > 0 {
+				mntSrc = groupConfig.Mounts[0].Source
+				mntDst = groupConfig.Mounts[0].Destination
+			}
+
+			for _, m := range ms {
+				if mntDst == "" && len(m.Config.Mounts) != 0 {
+					// TODO: Detaching a volume from a machine is possible, but it usually means a missconfiguration.
+					// We should show a warning and ask the user for confirmation and let it happen instead of failing here.
+					return fmt.Errorf(
+						"machine %s [%s] has a volume mounted but app config does not specify a volume; "+
+							"remove the volume from the machine or add a [mounts] section to fly.toml",
+						m.ID, groupName,
+					)
+				}
+
+				if mntDst != "" && len(m.Config.Mounts) == 0 {
+					// TODO: Attaching a volume to an existing machine is not possible, but it could replace the machine
+					// by another running on the same zone than the volume.
+					return fmt.Errorf(
+						"machine %s [%s] does not have a volume configured and fly.toml expects one with destination %s; "+
+							"remove the [mounts] configuration in fly.toml or use the machines API to add a volume to this machine",
+						m.ID, groupName, mntDst,
+					)
+				}
+
+				if mms := m.Config.Mounts; len(mms) > 0 && mntSrc != "" && mms[0].Name != "" && mntSrc != mms[0].Name {
+					// TODO: Changed the attached volume to an existing machine is not possible, but it could replace the machine
+					// by another running on the same zone than the new volume.
+					return fmt.Errorf(
+						"machine %s [%s] can't update the attached volume %s with name '%s' by '%s'",
+						m.ID, groupName, mntSrc, mms[0].Volume, mms[0].Name,
+					)
+				}
+			}
+
+		case false:
+			// Check if there are unattached volumes for new groups with mounts
+			for _, m := range groupConfig.Mounts {
+				if vs := md.volumes[m.Source]; len(vs) == 0 {
+					return fmt.Errorf(
+						"creating a new machine in group '%s' requires an unattached '%s' volume. Create it with `fly volume create %s`",
+						groupName, m.Source, m.Source)
+				}
+			}
 		}
 	}
 
-	if md.machineSet.IsEmpty() && volumeDestination != "" && len(md.volumes) == 0 {
-		return fmt.Errorf("error new machine requires an unattached volume named '%s' on mount destination '%s'",
-			md.appConfig.Mounts.Source, volumeDestination)
-	}
 	return nil
 }
 
@@ -337,66 +385,6 @@ func (md *machineDeployment) updateReleaseInBackend(ctx context.Context, status
 	return nil
 }
 
-func (md *machineDeployment) provisionIpsOnFirstDeploy(ctx context.Context) error {
-	// Deploy only if the app hasn't been deployed and have defined services
-	if md.app.Deployed || !md.machineSet.IsEmpty() || (md.appConfig.HttpService == nil && len(md.appConfig.Services) == 0) {
-		return nil
-	}
-
-	// Do not touch IPs if there are already allocated
-	ipAddrs, err := md.apiClient.GetIPAddresses(ctx, md.app.Name)
-	if err != nil {
-		return fmt.Errorf("error detecting ip addresses allocated to %s app: %w", md.app.Name, err)
-	}
-	if len(ipAddrs) > 0 {
-		return nil
-	}
-
-	switch md.appConfig.HasNonHttpAndHttpsStandardServices() {
-	case true:
-		hasUdpService := md.appConfig.HasUdpService()
-
-		ipStuffStr := "a dedicated ipv4 address"
-		if !hasUdpService {
-			ipStuffStr = "dedicated ipv4 and ipv6 addresses"
-		}
-
-		confirmDedicatedIp, err := prompt.Confirmf(ctx, "Would you like to allocate %s now?", ipStuffStr)
-		if confirmDedicatedIp && err == nil {
-			v4Dedicated, err := md.apiClient.AllocateIPAddress(ctx, md.app.Name, "v4", "", nil, "")
-			if err != nil {
-				return err
-			}
-			fmt.Fprintf(md.io.Out, "Allocated dedicated ipv4: %s\n", v4Dedicated.Address)
-
-			if !hasUdpService {
-				v6Dedicated, err := md.apiClient.AllocateIPAddress(ctx, md.app.Name, "v6", "", nil, "")
-				if err != nil {
-					return err
-				}
-				fmt.Fprintf(md.io.Out, "Allocated dedicated ipv6: %s\n", v6Dedicated.Address)
-			}
-		}
-
-	case false:
-		fmt.Fprintf(md.io.Out, "Provisioning ips for %s\n", md.colorize.Bold(md.app.Name))
-		v6Addr, err := md.apiClient.AllocateIPAddress(ctx, md.app.Name, "v6", "", nil, "")
-		if err != nil {
-			return fmt.Errorf("error allocating ipv6 after detecting first deploy and presence of services: %w", err)
-		}
-		fmt.Fprintf(md.io.Out, "  Dedicated ipv6: %s\n", v6Addr.Address)
-
-		v4Shared, err := md.apiClient.AllocateSharedIPAddress(ctx, md.app.Name)
-		if err != nil {
-			return fmt.Errorf("error allocating shared ipv4 after detecting first deploy and presence of services: %w", err)
-		}
-		fmt.Fprintf(md.io.Out, "  Shared ipv4: %s\n", v4Shared)
-		fmt.Fprintf(md.io.Out, "  Add a dedicated ipv4 with: fly ips allocate-v4\n")
-	}
-
-	return nil
-}
-
 func (md *machineDeployment) logClearLinesAbove(count int) {
 	if md.io.IsInteractive() {
 		builder := aec.EmptyBuilder
@@ -405,38 +393,31 @@ func (md *machineDeployment) logClearLinesAbove(count int) {
 	}
 }
 
-func determineAppConfigForMachines(ctx context.Context, envFromFlags []string, primaryRegion string) (cfg *appconfig.Config, err error) {
-	appNameFromContext := appconfig.NameFromContext(ctx)
-	if cfg = appconfig.ConfigFromContext(ctx); cfg == nil {
-		logger := logger.FromContext(ctx)
-		logger.Debug("no local app config detected for machines deploy; fetching from backend ...")
-
-		cfg, err = appconfig.FromRemoteApp(ctx, appNameFromContext)
-		if err != nil {
-			return nil, err
-		}
+func determineAppConfigForMachines(ctx context.Context, envFromFlags []string, primaryRegion string) (*appconfig.Config, error) {
+	appConfig := appconfig.ConfigFromContext(ctx)
+	if appConfig == nil {
+		return nil, fmt.Errorf("BUG: application configuration must come in the context, be sure to pass it before calling NewMachineDeployment")
 	}
 
 	if len(envFromFlags) > 0 {
 		var parsedEnv map[string]string
-		if parsedEnv, err = cmdutil.ParseKVStringsToMap(envFromFlags); err != nil {
-			err = fmt.Errorf("failed parsing environment: %w", err)
-
-			return
+		parsedEnv, err := cmdutil.ParseKVStringsToMap(envFromFlags)
+		if err != nil {
+			return nil, fmt.Errorf("failed parsing environment: %w", err)
 		}
-		cfg.SetEnvVariables(parsedEnv)
+		appConfig.SetEnvVariables(parsedEnv)
 	}
 
 	// deleting this block will result in machines not being deployed in the user selected region
 	if primaryRegion != "" {
-		cfg.PrimaryRegion = primaryRegion
+		appConfig.PrimaryRegion = primaryRegion
 	}
 
 	// Always prefer the app name passed via --app
-
-	if appNameFromContext != "" {
-		cfg.AppName = appNameFromContext
+	appName := appconfig.NameFromContext(ctx)
+	if appName != "" {
+		appConfig.AppName = appName
 	}
 
-	return
+	return appConfig, nil
 }
