@@ -31,12 +31,15 @@ func (md *machineDeployment) launchInputForLaunch(processGroup string, guest *ap
 	mConfig.Guest = guest
 	mConfig.Image = md.img
 	md.setMachineReleaseData(mConfig)
+	// Get the final process group and prevent empty string
+	processGroup = mConfig.ProcessGroup()
 
-	if len(mConfig.Mounts) != 0 {
-		if len(md.volumes) == 0 {
-			return nil, fmt.Errorf("New machine in group '%s' needs an unattached volume named '%s'", processGroup, mConfig.Mounts[0].Name)
+	if len(mConfig.Mounts) > 0 {
+		mount0 := &mConfig.Mounts[0]
+		if len(md.volumes[mount0.Name]) == 0 {
+			return nil, fmt.Errorf("New machine in group '%s' needs an unattached volume named '%s'", processGroup, mount0.Name)
 		}
-		mConfig.Mounts[0].Volume = md.volumes[0].ID
+		mount0.Volume = md.volumes[mount0.Name][0].ID
 	}
 
 	return &api.LaunchMachineInput{
@@ -57,6 +60,8 @@ func (md *machineDeployment) launchInputForUpdate(origMachineRaw *api.Machine) (
 	}
 	mConfig.Image = md.img
 	md.setMachineReleaseData(mConfig)
+	// Get the final process group and prevent empty string
+	processGroup = mConfig.ProcessGroup()
 
 	// Mounts needs special treatment:
 	//   * Volumes attached to existings machines can't be swapped by other volumes
@@ -80,10 +85,10 @@ func (md *machineDeployment) launchInputForUpdate(origMachineRaw *api.Machine) (
 			// As we can't change the volume for a running machine, the only
 			// way is to destroy the current machine and launch a new one with the new volume attached
 			terminal.Warnf("Machine %s has volume '%s' attached but fly.toml have a different name: '%s'\n", mID, oMounts[0].Name, mMounts[0].Name)
-			if len(md.volumes) == 0 {
+			if len(md.volumes[mMounts[0].Name]) == 0 {
 				return nil, fmt.Errorf("machine in group '%s' needs an unattached volume named '%s'", processGroup, mMounts[0].Name)
 			}
-			mMounts[0].Volume = md.volumes[0].ID
+			mMounts[0].Volume = md.volumes[mMounts[0].Name][0].ID
 			mID = "" // Forces machine replacement
 		case mMounts[0].Path != oMounts[0].Path:
 			// The volume is the same but its mount path changed. Not a big deal.
@@ -101,10 +106,11 @@ func (md *machineDeployment) launchInputForUpdate(origMachineRaw *api.Machine) (
 		// Replace the machine because [mounts] section was added to fly.toml
 		// and it is not possible to attach a volume to an existing machine.
 		// The volume could be in a different zone than the machine.
-		if len(md.volumes) == 0 {
+		mount0 := &mMounts[0]
+		if len(md.volumes[mount0.Name]) == 0 {
 			return nil, fmt.Errorf("machine in group '%s' needs an unattached volume named '%s'", processGroup, mMounts[0].Name)
 		}
-		mMounts[0].Volume = md.volumes[0].ID
+		mount0.Volume = md.volumes[mount0.Name][0].ID
 		mID = "" // Forces machine replacement
 	}
 
