@@ -19,7 +19,6 @@ import (
 	"github.com/superfly/flyctl/internal/command/apps"
 	"github.com/superfly/flyctl/internal/state"
 	"github.com/superfly/flyctl/iostreams"
-	"github.com/superfly/flyctl/terminal"
 )
 
 type AppChecker struct {
@@ -33,29 +32,23 @@ type AppChecker struct {
 	apiClient  *api.Client
 }
 
-func NewAppChecker(ctx context.Context, jsonOutput bool, color *iostreams.ColorScheme) *AppChecker {
+func NewAppChecker(ctx context.Context, jsonOutput bool, color *iostreams.ColorScheme) (*AppChecker, error) {
 	appName := appconfig.NameFromContext(ctx)
 	if appName == "" {
 		if !jsonOutput {
 			fmt.Println("No app provided; skipping app specific checks")
 		}
-		return nil
+		return nil, nil
 	}
 
 	apiClient := client.FromContext(ctx).API()
 	appCompact, err := apiClient.GetAppCompact(ctx, appName)
 	if err != nil {
-		if !jsonOutput {
-			terminal.Debugf("API error looking up app with name %s: %v\n", appName, err)
-		}
-		return nil
+		return nil, err
 	}
 	ctx, err = apps.BuildContext(ctx, appCompact)
 	if err != nil {
-		if !jsonOutput {
-			terminal.Debugf("error building context for app %s: %v\n", appName, err)
-		}
-		return nil
+		return nil, err
 	}
 
 	ac := &AppChecker{
@@ -71,12 +64,20 @@ func NewAppChecker(ctx context.Context, jsonOutput bool, color *iostreams.ColorS
 
 	if !appCompact.Deployed && appCompact.PlatformVersion != "machines" {
 		ac.lprint(color.Yellow, "%s app has not been deployed yet. Skipping app checks. Deploy using `flyctl deploy`.\n", appName)
-		return nil
+		return nil, nil
 	}
 
 	ac.app = appCompact
 	ac.appConfig = appconfig.ConfigFromContext(ctx)
-	return ac
+
+	if ac.appConfig == nil {
+		ac.appConfig, err = appconfig.FromRemoteApp(ctx, ac.app.Name)
+		if err != nil {
+			return nil, err
+		}
+	}
+
+	return ac, nil
 }
 
 func (ac *AppChecker) lprint(color func(string) string, fmtstr string, args ...interface{}) {
