@@ -4,11 +4,8 @@ import (
 	"context"
 	"fmt"
 	"path/filepath"
-	"regexp"
 	"strings"
 
-	"github.com/cavaliergopher/grab/v3"
-	"github.com/logrusorgru/aurora"
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
 	"github.com/superfly/flyctl/client"
@@ -59,6 +56,11 @@ func New() (cmd *cobra.Command) {
 			Description: "Use the configuration file if present without prompting",
 			Default:     false,
 		},
+		flag.Bool{
+			Name:        "reuse-app",
+			Description: "Continue even if app name clashes with an existent app",
+			Default:     false,
+		},
 		flag.Bool{
 			Name:        "dockerignore-from-gitignore",
 			Description: "If a .dockerignore does not exist, create one from .gitignore files",
@@ -78,12 +80,6 @@ func run(ctx context.Context) (err error) {
 	io := iostreams.FromContext(ctx)
 	client := client.FromContext(ctx).API()
 	workingDir := flag.GetString(ctx, "path")
-	existingConfig := appconfig.ConfigFromContext(ctx)
-	generateName := flag.GetBool(ctx, "generate-name")
-	copyConfig := flag.GetBool(ctx, "copy-config")
-	name := strings.TrimSpace(flag.GetString(ctx, "name"))
-	appConfig := appconfig.NewConfig()
-	launchIntoExistingApp := false
 
 	deployArgs := deploy.DeployWithConfigArgs{
 		ForceNomad:    flag.GetBool(ctx, "force-nomad"),
@@ -95,130 +91,29 @@ func run(ctx context.Context) (err error) {
 	if absDir, err := filepath.Abs(workingDir); err == nil {
 		workingDir = absDir
 	}
-
 	configFilePath := filepath.Join(workingDir, appconfig.DefaultConfigFileName)
-
-	if existingConfig != nil {
-		if existingConfig.AppName != "" {
-			fmt.Fprintln(io.Out, "An existing fly.toml file was found for app", existingConfig.AppName)
-		} else {
-			fmt.Fprintln(io.Out, "An existing fly.toml file was found")
-		}
-
-		if !copyConfig {
-			copy, err := prompt.Confirm(ctx, "Would you like to copy its configuration to the new app?")
-			if err != nil {
-				return err
-			}
-
-			copyConfig = copy
-		}
-
-		if copyConfig {
-			appConfig = existingConfig
-		}
-	}
-
 	fmt.Fprintln(io.Out, "Creating app in", workingDir)
 
-	srcInfo := new(scanner.SourceInfo)
-	config := &scanner.ScannerConfig{
-		ExistingPort: appConfig.InternalPort(),
+	appConfig, copyConfig, err := determineBaseAppConfig(ctx)
+	if err != nil {
+		return err
 	}
 
-	// Detect if --copy-config and --now flags are set. If so, limited set of
-	// fly.toml file updates. Helpful for deploying PRs when the project is
-	// already setup and we only need fly.toml config changes.
-	if flag.GetBool(ctx, "copy-config") && flag.GetBool(ctx, "now") {
-		config.Mode = "clone"
-	} else {
-		config.Mode = "launch"
+	var srcInfo *scanner.SourceInfo
+	srcInfo, appConfig.Build, err = determineSourceInfo(ctx, appConfig, copyConfig, workingDir)
+	if err != nil {
+		return err
 	}
 
-	if img := flag.GetString(ctx, "image"); img != "" {
-		fmt.Fprintln(io.Out, "Using image", img)
-		appConfig.Build = &appconfig.Build{
-			Image: img,
-		}
-	} else if dockerfile := flag.GetString(ctx, "dockerfile"); dockerfile != "" {
-		if strings.HasPrefix(dockerfile, "http://") || strings.HasPrefix(dockerfile, "https://") {
-			fmt.Fprintln(io.Out, "Downloading dockerfile", dockerfile)
-			resp, err := grab.Get("Dockerfile", dockerfile)
-			if err != nil {
-				return err
-			} else {
-				appConfig.Build = &appconfig.Build{
-					Dockerfile: resp.Filename,
-				}
-
-				// scan Dockerfile for port
-				if si, err := scanner.Scan(workingDir, config); err != nil {
-					return err
-				} else {
-					srcInfo = si
-				}
-			}
-		} else {
-			fmt.Fprintln(io.Out, "Using dockerfile", dockerfile)
-			appConfig.Build = &appconfig.Build{
-				Dockerfile: dockerfile,
-			}
-		}
-	} else {
-		fmt.Fprintln(io.Out, "Scanning source code")
-		if si, err := scanner.Scan(workingDir, config); err != nil {
-			return err
-		} else {
-			srcInfo = si
-		}
-
-		if srcInfo == nil {
-			fmt.Fprintln(io.Out, aurora.Green("Could not find a Dockerfile, nor detect a runtime or framework from source code. Continuing with a blank app."))
-		} else {
-			var article string = "a"
-			if matched, _ := regexp.MatchString(`^[aeiou]`, strings.ToLower(srcInfo.Family)); matched {
-				article += "n"
-			}
-
-			appType := srcInfo.Family
-			if srcInfo.Version != "" {
-				appType = appType + " " + srcInfo.Version
-			}
-
-			fmt.Fprintf(io.Out, "Detected %s %s app\n", article, aurora.Green(appType))
-
-			if srcInfo.Builder != "" {
-				fmt.Fprintln(io.Out, "Using the following build configuration:")
-				fmt.Fprintln(io.Out, "\tBuilder:", srcInfo.Builder)
-				if srcInfo.Buildpacks != nil && len(srcInfo.Buildpacks) > 0 {
-					fmt.Fprintln(io.Out, "\tBuildpacks:", strings.Join(srcInfo.Buildpacks, " "))
-				}
-
-				appConfig.Build = &appconfig.Build{
-					Builder:    srcInfo.Builder,
-					Buildpacks: srcInfo.Buildpacks,
-				}
-			}
-		}
-	}
-
-	if generateName {
-		appConfig.AppName = ""
-	}
-
-	if name != "" {
-		appConfig.AppName = name
-	}
-
-	if !generateName && name == "" {
-		inputName, err := promptForAppName(ctx, appConfig)
-		if err != nil {
-			return err
-		}
-		appConfig.AppName = inputName
+	appConfig.AppName, err = determineAppName(ctx, appConfig)
+	if err != nil {
+		return err
 	}
 
 	var org *api.Organization
+	var existingAppPlatform string
+	var launchIntoExistingApp bool
+
 	if appConfig.AppName != "" {
 		exists, app, err := appExists(ctx, appConfig)
 		if err != nil {
@@ -226,29 +121,31 @@ func run(ctx context.Context) (err error) {
 		}
 
 		if exists {
-			msg := fmt.Sprintf("App %s already exists, do you want to launch into that app?", appConfig.AppName)
-			launchIntoExistingApp, err = prompt.Confirm(ctx, msg)
-			if err != nil {
-				return err
-			}
-			if !launchIntoExistingApp {
-				return nil
+			launchIntoExistingApp = flag.GetBool(ctx, "reuse-app")
+			if !flag.IsSpecified(ctx, "reuse-app") {
+				msg := fmt.Sprintf("App %s already exists, do you want to launch into that app?", appConfig.AppName)
+				launchIntoExistingApp, err = prompt.Confirm(ctx, msg)
+				if err != nil {
+					return err
+				}
+				if !launchIntoExistingApp {
+					return nil
+				}
 			}
 
+			existingAppPlatform = app.PlatformVersion
+
 			org = &api.Organization{
 				ID:       app.Organization.ID,
 				Name:     app.Organization.Name,
 				Slug:     app.Organization.Slug,
 				PaidPlan: app.Organization.PaidPlan,
 			}
-
 		}
-
 	}
 
-	if !launchIntoExistingApp {
-		// Prompt for an org
-		// TODO: determine if eager remote builder is still required here
+	// Prompt for an org
+	if org == nil {
 		org, err = prompt.Org(ctx)
 		if err != nil {
 			return
@@ -257,54 +154,63 @@ func run(ctx context.Context) (err error) {
 
 	// If we potentially are deploying, launch a remote builder to prepare for deployment.
 	if !flag.GetBool(ctx, "no-deploy") {
+		// TODO: determine if eager remote builder is still required here
 		go imgsrc.EagerlyEnsureRemoteBuilder(ctx, client, org.Slug)
 	}
 
-	region, err := prompt.Region(ctx, !org.PaidPlan, prompt.RegionParams{
-		Message: "Choose a region for deployment:",
-	})
+	region, err := computeRegionToUse(ctx, appConfig, org.PaidPlan)
 	if err != nil {
 		return err
 	}
+	// Do not change PrimaryRegion after this line
 	appConfig.PrimaryRegion = region.Code
+	fmt.Fprintf(io.Out, "App will use '%s' region as primary\n", appConfig.PrimaryRegion)
 
-	shouldUseMachines, err := shouldAppUseMachinesPlatform(ctx, org.Slug)
+	shouldUseMachines, err := shouldAppUseMachinesPlatform(ctx, org.Slug, existingAppPlatform)
 	if err != nil {
 		return err
 	}
 
-	if shouldUseMachines && copyConfig {
+	if copyConfig && shouldUseMachines {
 		// Check imported fly.toml is a valid V2 config before creating the app
 		if err := appConfig.SetMachinesPlatform(); err != nil {
 			return fmt.Errorf("Can not use configuration for Apps V2, check fly.toml: %w", err)
 		}
 	}
 
-	if !launchIntoExistingApp {
-		input := api.CreateAppInput{
+	switch {
+	// App exists and it is not importing existing fly.toml
+	case launchIntoExistingApp && !copyConfig:
+		appConfig, err = freshV2Config(appConfig.AppName, appConfig)
+		if err != nil {
+			return err
+		}
+	// App doesn't exist, just create a new app
+	case !launchIntoExistingApp:
+		createdApp, err := client.CreateApp(ctx, api.CreateAppInput{
 			Name:            appConfig.AppName,
 			OrganizationID:  org.ID,
 			PreferredRegion: &appConfig.PrimaryRegion,
 			Machines:        shouldUseMachines,
-		}
-
-		createdApp, err := client.CreateApp(ctx, input)
+		})
 		if err != nil {
 			return err
 		}
 
-		if !copyConfig {
-			// Use the default configuration template suggested by Web
-			newCfg, err := appconfig.FromDefinition(&createdApp.Config.Definition)
-			if err != nil {
-				return fmt.Errorf("Launch failed to get new app configuration: %w", err)
-			}
-			newCfg.AppName = createdApp.Name
-			newCfg.Build = appConfig.Build
-			newCfg.PrimaryRegion = appConfig.PrimaryRegion
-			appConfig = newCfg
-		} else {
+		switch {
+		case copyConfig:
 			appConfig.AppName = createdApp.Name
+		case shouldUseMachines:
+			appConfig, err = freshV2Config(createdApp.Name, appConfig)
+			if err != nil {
+				return fmt.Errorf("failed to create new V2 app configuration: %w", err)
+			}
+		default:
+			// Use the default configuration template suggested by Web
+			appConfig, err = freshV1Config(createdApp.Name, appConfig, &createdApp.Config.Definition)
+			if err != nil {
+				return fmt.Errorf("failed to get new configuration: %w", err)
+			}
 		}
 		fmt.Fprintf(io.Out, "Created app '%s' in organization '%s'\n", appConfig.AppName, org.Slug)
 	}
@@ -321,7 +227,7 @@ func run(ctx context.Context) (err error) {
 		return err
 	}
 	// If volumes are requested by the launch scanner, create them
-	if err := createVolumes(ctx, srcInfo, appConfig.AppName, region.Code); err != nil {
+	if err := createVolumes(ctx, srcInfo, appConfig.AppName, appConfig.PrimaryRegion); err != nil {
 		return err
 	}
 	// If database are requested by the launch scanner, create them
@@ -350,11 +256,6 @@ func run(ctx context.Context) (err error) {
 		appConfig.SetInternalPort(n)
 	}
 
-	// remove auto-rollback from machine fly.tomls
-	if shouldUseMachines {
-		appConfig.Experimental = nil
-	}
-
 	// Finally write application configuration to fly.toml
 	if err := appConfig.WriteToDisk(ctx, configFilePath); err != nil {
 		return err
@@ -406,13 +307,32 @@ func run(ctx context.Context) (err error) {
 	return nil
 }
 
-func shouldAppUseMachinesPlatform(ctx context.Context, orgSlug string) (bool, error) {
-	apiClient := client.FromContext(ctx).API()
-	if flag.GetBool(ctx, "force-machines") {
+func shouldAppUseMachinesPlatform(ctx context.Context, orgSlug, existingAppPlatform string) (bool, error) {
+	forceMachines := flag.GetBool(ctx, "force-machines")
+	forceNomad := flag.GetBool(ctx, "force-nomad")
+
+	// Keep if we are reusing an app and it has platform version set
+	if existingAppPlatform != "" {
+		isV2 := existingAppPlatform == appconfig.MachinesPlatform
+		switch {
+		case forceMachines && !isV2:
+			return false, fmt.Errorf("--force-machines won't work for existing app in nomad platform")
+		case forceNomad && isV2:
+			return false, fmt.Errorf("--force-nomad won't work for existing app in machines platform")
+		default:
+			return isV2, nil
+		}
+	}
+
+	// Otherwise looks for CLI flags and organization defaults
+	if forceMachines {
 		return true, nil
-	} else if flag.GetBool(ctx, "force-nomad") {
+	} else if forceNomad {
 		return false, nil
 	}
+
+	// Query the organization looking for default platform version to use
+	apiClient := client.FromContext(ctx).API()
 	orgDefault, err := apiClient.GetAppsV2DefaultOnForOrg(ctx, orgSlug)
 	if err != nil {
 		return false, err
@@ -451,3 +371,57 @@ func promptForAppName(ctx context.Context, cfg *appconfig.Config) (name string,
 
 	return
 }
+
+// determineBaseAppConfig looks for existing app config, ask to reuse or returns an empty config
+func determineBaseAppConfig(ctx context.Context) (*appconfig.Config, bool, error) {
+	io := iostreams.FromContext(ctx)
+
+	existingConfig := appconfig.ConfigFromContext(ctx)
+	if existingConfig != nil {
+
+		if existingConfig.AppName != "" {
+			fmt.Fprintln(io.Out, "An existing fly.toml file was found for app", existingConfig.AppName)
+		} else {
+			fmt.Fprintln(io.Out, "An existing fly.toml file was found")
+		}
+
+		copyConfig := flag.GetBool(ctx, "copy-config")
+		if !flag.IsSpecified(ctx, "copy-config") {
+			var err error
+			copyConfig, err = prompt.Confirm(ctx, "Would you like to copy its configuration to the new app?")
+			switch {
+			case prompt.IsNonInteractive(err) && !flag.GetBool(ctx, "auto-confirm"):
+				return nil, false, err
+			case err != nil:
+				return nil, false, err
+			}
+		}
+
+		if copyConfig {
+			return existingConfig, true, nil
+		}
+	}
+
+	return appconfig.NewConfig(), false, nil
+}
+
+func determineAppName(ctx context.Context, appConfig *appconfig.Config) (string, error) {
+	generateName := flag.GetBool(ctx, "generate-name")
+	if generateName {
+		return "", nil
+	}
+
+	name := strings.TrimSpace(flag.GetString(ctx, "name"))
+	if name != "" {
+		return name, nil
+	}
+
+	switch name, err := promptForAppName(ctx, appConfig); {
+	case prompt.IsNonInteractive(err):
+		return "", fmt.Errorf("--name or --generate-name flags must be specified when not running interactively")
+	case err != nil:
+		return "", err
+	default:
+		return name, nil
+	}
+}
