@@ -60,11 +60,6 @@ var sharedFlags = flag.Set{
 		Shorthand:   "e",
 		Description: "Set of environment variables in the form of NAME=VALUE pairs. Can be specified multiple times.",
 	},
-	flag.StringSlice{
-		Name:        "volume",
-		Shorthand:   "v",
-		Description: "Volumes to mount in the form of <volume_id_or_name>:/path/inside/machine[:<options>]",
-	},
 	flag.String{
 		Name:        "entrypoint",
 		Description: "ENTRYPOINT replacement",
@@ -176,6 +171,11 @@ func newRun() *cobra.Command {
 			Name:        "rm",
 			Description: "Automatically remove the machine when it exits",
 		},
+		flag.StringSlice{
+			Name:        "volume",
+			Shorthand:   "v",
+			Description: "Volumes to mount in the form of <volume_id_or_name>:/path/inside/machine[:<options>]",
+		},
 		sharedFlags,
 	)
 
@@ -758,7 +758,10 @@ func determineMachineConfig(ctx context.Context, input *determineMachineConfigIn
 		return machineConf, err
 	}
 	machineConf.Image = img.Tag
-	machineConf.DisableMachineAutostart = api.Pointer(!flag.GetBool(ctx, "autostart"))
+
+	if flag.IsSpecified(ctx, "autostart") {
+		machineConf.DisableMachineAutostart = api.Pointer(!flag.GetBool(ctx, "autostart"))
+	}
 
 	return machineConf, nil
 }
