@@ -7,19 +7,13 @@ import (
 	"os"
 	"os/signal"
 	"runtime"
-	"strconv"
 	"strings"
 	"sync/atomic"
 	"syscall"
 	"time"
 
-	"github.com/superfly/flyctl/internal/command/apps"
-	"golang.org/x/exp/slices"
-
 	"github.com/AlecAivazis/survey/v2"
 	"github.com/Khan/genqlient/graphql"
-	"github.com/briandowns/spinner"
-	"github.com/jpillora/backoff"
 	"github.com/samber/lo"
 	"github.com/spf13/cobra"
 	"github.com/superfly/flyctl/api"
@@ -28,6 +22,7 @@ import (
 	"github.com/superfly/flyctl/gql"
 	"github.com/superfly/flyctl/internal/appconfig"
 	"github.com/superfly/flyctl/internal/command"
+	"github.com/superfly/flyctl/internal/command/apps"
 	"github.com/superfly/flyctl/internal/command/deploy"
 	"github.com/superfly/flyctl/internal/flag"
 	"github.com/superfly/flyctl/internal/machine"
@@ -37,6 +32,7 @@ import (
 	"github.com/superfly/flyctl/internal/state"
 	"github.com/superfly/flyctl/iostreams"
 	"github.com/superfly/flyctl/terminal"
+	"golang.org/x/exp/slices"
 )
 
 func New() *cobra.Command {
@@ -58,6 +54,11 @@ func newMigrateToV2() *cobra.Command {
 		flag.Yes(),
 		flag.App(),
 		flag.AppConfig(),
+		flag.Bool{
+			// Note: This is referenced in a preflight test, make sure to remove that when removing this flag!
+			Name:        "experimental-volume-migration",
+			Description: "Migrate apps with volumes. Causes brief downtime. Experimental feature, use at your own risk!",
+		},
 		flag.String{
 			Name:        "primary-region",
 			Description: "Specify primary region if one is not set in fly.toml",
@@ -112,6 +113,7 @@ type V2PlatformMigrator interface {
 type NewVolume struct {
 	vol             *api.Volume
 	previousAllocId string
+	mountPoint      string
 }
 
 // FIXME: a lot of stuff is shared with MachineDeployment... might be useful to consolidate the shared stuff into another library/package/something
@@ -128,7 +130,6 @@ type v2PlatformMigrator struct {
 	appConfig               *appconfig.Config
 	configPath              string
 	autoscaleConfig         *api.AutoscalingConfig
-	volumeDestination       string
 	formattedProcessConfigs string
 	img                     string
 	appLock                 string
@@ -139,10 +140,11 @@ type v2PlatformMigrator struct {
 	oldVmCounts             map[string]int
 	newMachinesInput        []*api.LaunchMachineInput
 	newMachines             machine.MachineSet
-	canAvoidDowntime        bool
 	recovery                recoveryState
-	isPostgres              bool
+	usesForkedVolumes       bool
 	createdVolumes          []*NewVolume
+	replacedVolumes         map[string]int
+	isPostgres              bool
 	pgConsulUrl             string
 }
 
@@ -179,6 +181,10 @@ func NewV2PlatformMigrator(ctx context.Context, appName string) (V2PlatformMigra
 	if err != nil {
 		return nil, err
 	}
+	formattedProcessConfigs := appConfig.FormatProcessNames()
+	if err != nil {
+		return nil, err
+	}
 	autoscaleConfig, err := apiClient.AppAutoscalingConfig(ctx, appName)
 	if err != nil {
 		return nil, err
@@ -214,13 +220,12 @@ func NewV2PlatformMigrator(ctx context.Context, appName string) (V2PlatformMigra
 		appFull:                 appFull,
 		appConfig:               appConfig,
 		autoscaleConfig:         autoscaleConfig,
-		volumeDestination:       appConfig.MountsDestination(),
-		formattedProcessConfigs: appConfig.FormatProcessNames(),
+		formattedProcessConfigs: formattedProcessConfigs,
 		img:                     img,
 		oldAllocs:               allocs,
 		machineGuest:            machineGuest,
-		canAvoidDowntime:        true,
 		isPostgres:              appCompact.IsPostgresApp(),
+		replacedVolumes:         map[string]int{},
 		recovery: recoveryState{
 			platformVersion: appFull.PlatformVersion,
 		},
@@ -287,8 +292,9 @@ func (m *v2PlatformMigrator) rollback(ctx context.Context, tb *render.TextBlock)
 		}
 	}
 	if len(m.createdVolumes) > 0 {
+		tb.Detailf("Removing migration-created volumes")
 		for _, vol := range m.createdVolumes {
-			_, err := m.apiClient.DeleteVolume(ctx, vol.vol.ID)
+			_, err := m.apiClient.DeleteVolume(ctx, vol.vol.ID, m.appLock)
 			if err != nil {
 				return err
 			}
@@ -309,6 +315,7 @@ func (m *v2PlatformMigrator) rollback(ctx context.Context, tb *render.TextBlock)
 			return err
 		}
 	}
+
 	if m.recovery.scaledToZero && len(m.oldAllocs) > 0 {
 		// Restore nomad allocs
 		tb.Detail("Restoring nomad allocs to their previous state")
@@ -402,7 +409,7 @@ func (m *v2PlatformMigrator) Migrate(ctx context.Context) (err error) {
 	}
 
 	tb.Detail("Locking app to prevent changes during the migration")
-	err = m.lockAppForMigration(ctx)
+	err = m.lockApp(ctx)
 	if err != nil {
 		return err
 	}
@@ -410,7 +417,18 @@ func (m *v2PlatformMigrator) Migrate(ctx context.Context) (err error) {
 		return abortedErr
 	}
 
-	if !m.canAvoidDowntime {
+	if !m.isPostgres {
+		tb.Detail("Making snapshots of volumes for the new machines")
+		err = m.migrateAppVolumes(ctx)
+		if err != nil {
+			return err
+		}
+		if aborted.Load() {
+			return abortedErr
+		}
+	}
+
+	if m.requiresDowntime() {
 		tb.Detail("Scaling down to zero VMs. This will cause temporary downtime until new VMs come up.")
 
 		err = m.scaleNomadToZero(ctx)
@@ -508,8 +526,8 @@ func (m *v2PlatformMigrator) Migrate(ctx context.Context) (err error) {
 		}
 	}
 
-	if m.canAvoidDowntime {
-		tb.Detail("Scaling down to zero nomad VMs now that machines are running.")
+	if !m.requiresDowntime() {
+		tb.Detail("Scaling nomad VMs down to zero now that machines are running.")
 
 		err = m.scaleNomadToZero(ctx)
 		if err != nil {
@@ -546,17 +564,18 @@ func (m *v2PlatformMigrator) Migrate(ctx context.Context) (err error) {
 	}
 
 	tb.Done("Done")
+	m.printReplacedVolumes()
+
 	return nil
 }
 
 func (m *v2PlatformMigrator) inRegionMachines() []*api.Machine {
-	var machines []*api.Machine
-	for _, machine := range m.newMachines.GetMachines() {
-		if machine.Machine().Region == m.appConfig.PrimaryRegion {
-			machines = append(machines, machine.Machine())
+	return lo.FilterMap(m.newMachines.GetMachines(), func(mach machine.LeasableMachine, _ int) (*api.Machine, bool) {
+		if mach.Machine().Region == m.appConfig.PrimaryRegion {
+			return mach.Machine(), true
 		}
-	}
-	return machines
+		return nil, false
+	})
 }
 
 func (m *v2PlatformMigrator) validate(ctx context.Context) error {
@@ -595,24 +614,6 @@ func (m *v2PlatformMigrator) validateScaling(ctx context.Context) error {
 	return nil
 }
 
-func (m *v2PlatformMigrator) validateVolumes(ctx context.Context) error {
-	// FIXME: for now we fail if there are volumes... remove this once we figure out volumes
-	// When we can migrate apps with volumes, you probably want to set `m.canAvoidDowntime`
-	// to false when the app has volumes.
-	if m.isPostgres {
-		return nil
-	}
-	if m.appConfig.Mounts != nil {
-		return fmt.Errorf("cannot migrate app %s with [mounts] configuration, yet; watch https://community.fly.io for announcements about volume support with migrations", m.appCompact.Name)
-	}
-	for _, a := range m.oldAllocs {
-		if len(a.AttachedVolumes.Nodes) > 0 {
-			return fmt.Errorf("cannot migrate app %s because alloc %s has a volume attached; watch https://community.fly.io for announcements about volume support with migrations", m.appCompact.Name, a.IDShort)
-		}
-	}
-	return nil
-}
-
 func (m *v2PlatformMigrator) validateProcessGroupsOnAllocs(ctx context.Context) error {
 	validGroupNames := m.appConfig.ProcessNames()
 	for _, a := range m.oldAllocs {
@@ -623,28 +624,6 @@ func (m *v2PlatformMigrator) validateProcessGroupsOnAllocs(ctx context.Context)
 	return nil
 }
 
-func (m *v2PlatformMigrator) lockAppForMigration(ctx context.Context) error {
-	_ = `# @genqlient
-	mutation LockApp($input:LockAppInput!) {
-        lockApp(input:$input) {
-			lockId
-			expiration
-        }
-	}
-	`
-	input := gql.LockAppInput{
-		AppId: m.appConfig.AppName,
-	}
-	resp, err := gql.LockApp(ctx, m.gqlClient, input)
-	if err != nil {
-		return err
-	}
-
-	m.appLock = resp.LockApp.LockId
-	m.recovery.appLocked = true
-	return nil
-}
-
 func (m *v2PlatformMigrator) createRelease(ctx context.Context) error {
 	_ = `# @genqlient
 	mutation MigrateMachinesCreateRelease($input:CreateReleaseInput!) {
@@ -679,61 +658,6 @@ func (m *v2PlatformMigrator) resolveProcessGroups(ctx context.Context) {
 	}
 }
 
-func (m *v2PlatformMigrator) scaleNomadToZero(ctx context.Context) error {
-	input := gql.SetVMCountInput{
-		AppId:  m.appConfig.AppName,
-		LockId: m.appLock,
-		GroupCounts: lo.MapToSlice(m.oldVmCounts, func(name string, count int) gql.VMCountInput {
-			return gql.VMCountInput{Group: name, Count: 0}
-		}),
-	}
-
-	if len(input.GroupCounts) > 0 {
-
-		_, err := gql.SetNomadVMCount(ctx, m.gqlClient, input)
-		if err != nil {
-			return err
-		}
-	}
-	err := m.waitForAllocsZero(ctx)
-	if err != nil {
-		return err
-	}
-
-	m.recovery.scaledToZero = true
-	return nil
-}
-
-func (m *v2PlatformMigrator) waitForAllocsZero(ctx context.Context) error {
-	s := spinner.New(spinner.CharSets[9], 200*time.Millisecond)
-	s.Writer = m.io.ErrOut
-	s.Prefix = fmt.Sprintf("Waiting for nomad allocs for '%s' to be destroyed ", m.appCompact.Name)
-	s.Start()
-	defer s.Stop()
-
-	timeout := time.After(1 * time.Hour)
-	b := &backoff.Backoff{
-		Min:    2 * time.Second,
-		Max:    5 * time.Minute,
-		Factor: 1.2,
-		Jitter: true,
-	}
-	for {
-		select {
-		case <-time.After(b.Duration()):
-			currentAllocs, err := m.apiClient.GetAllocations(ctx, m.appCompact.Name, false)
-			if err != nil {
-				return err
-			}
-			if len(currentAllocs) == 0 {
-				return nil
-			}
-		case <-timeout:
-			return errors.New("nomad allocs never reached zero, timed out")
-		}
-	}
-}
-
 func (m *v2PlatformMigrator) updateAppPlatformVersion(ctx context.Context, platform string) error {
 	_ = `# @genqlient
 	mutation SetPlatformVersion($input:SetPlatformVersionInput!) {
@@ -755,64 +679,18 @@ func (m *v2PlatformMigrator) updateAppPlatformVersion(ctx context.Context, platf
 	return nil
 }
 
-func (m *v2PlatformMigrator) createMachines(ctx context.Context) error {
-	var newlyCreatedMachines []*api.Machine
-	defer func() {
-		m.recovery.machinesCreated = newlyCreatedMachines
-	}()
-
-	for _, machineInput := range m.newMachinesInput {
-		if exists, vol := m.volumeForPrevAlloc(machineInput.Config.Metadata["prevAlloc"]); exists {
-			machineInput.Config.Mounts = []api.MachineMount{{
-				Name:   "pg_data_machines",
-				Path:   "/data",
-				Volume: vol.ID,
-			}}
-		}
-		newMachine, err := m.flapsClient.Launch(ctx, *machineInput)
-		if err != nil {
-			return fmt.Errorf("failed creating a machine in region %s: %w", machineInput.Region, err)
-		}
-		newlyCreatedMachines = append(newlyCreatedMachines, newMachine)
-	}
-	for _, mach := range newlyCreatedMachines {
-		err := machine.WaitForStartOrStop(ctx, mach, "start", time.Minute*5)
-		if err != nil {
-			return err
-		}
-	}
-	m.newMachines = machine.NewMachineSet(m.flapsClient, m.io, newlyCreatedMachines)
-	return nil
-}
-
-func (m *v2PlatformMigrator) unlockApp(ctx context.Context) error {
-	_ = `# @genqlient
-	mutation UnlockApp($input:UnlockAppInput!) {
-		unlockApp(input:$input) {
-			app { id }
-		}
-	}
-	`
-	input := gql.UnlockAppInput{
-		AppId:  m.appConfig.AppName,
-		LockId: m.appLock,
-	}
-	_, err := gql.UnlockApp(ctx, m.gqlClient, input)
-	if err != nil {
-		return err
-	}
-	m.recovery.appLocked = false
-	return nil
-}
-
 func (m *v2PlatformMigrator) deployApp(ctx context.Context) error {
 	input := deploy.MachineDeploymentArgs{
 		AppCompact:  m.appCompact,
 		RestartOnly: true,
 	}
 	if m.isPostgres {
-		input.NewVolumeName = "pg_data_machines"
+		if len(m.appConfig.Mounts) > 0 {
+			m.appConfig.Mounts[0].Source = "pg_data_machines"
+		}
 	}
+	// Feed appConfig into the context so MachineDeployment can reuse it
+	ctx = appconfig.WithConfig(ctx, m.appConfig)
 	md, err := deploy.NewMachineDeployment(ctx, input)
 	if err != nil {
 		sentry.CaptureExceptionWithAppInfo(err, "migrate-to-v2", m.appCompact)
@@ -825,76 +703,8 @@ func (m *v2PlatformMigrator) deployApp(ctx context.Context) error {
 	return nil
 }
 
-func (m *v2PlatformMigrator) prepMachinesToCreate(ctx context.Context) error {
-	var err error
-	m.newMachinesInput, err = m.resolveMachinesFromAllocs()
-	// FIXME: add extra machines that are stopped by default, based on scaling/autoscaling config for the app
-	return err
-}
-
-func (m *v2PlatformMigrator) resolveMachinesFromAllocs() ([]*api.LaunchMachineInput, error) {
-	var res []*api.LaunchMachineInput
-	for _, alloc := range m.oldAllocs {
-		mach, err := m.resolveMachineFromAlloc(alloc)
-		if err != nil {
-			return nil, err
-		}
-		res = append(res, mach)
-	}
-	return res, nil
-}
-
-func (m *v2PlatformMigrator) volumeForPrevAlloc(id string) (bool, *api.Volume) {
-	for _, vol := range m.createdVolumes {
-		if vol.previousAllocId == id {
-			return true, vol.vol
-		}
-	}
-	return false, nil
-}
-
-func (m *v2PlatformMigrator) resolveMachineFromAlloc(alloc *api.AllocationStatus) (*api.LaunchMachineInput, error) {
-	mConfig, err := m.appConfig.ToMachineConfig(alloc.TaskName, nil)
-	if err != nil {
-		return nil, err
-	}
-
-	mConfig.Guest = m.machineGuest
-	mConfig.Image = m.img
-	mConfig.Metadata[api.MachineConfigMetadataKeyFlyReleaseId] = m.releaseId
-	mConfig.Metadata[api.MachineConfigMetadataKeyFlyReleaseVersion] = strconv.Itoa(m.releaseVersion)
-	mConfig.Metadata["prevAlloc"] = alloc.ID
-
-	if m.isPostgres {
-		mConfig.Env["FLY_CONSUL_URL"] = m.pgConsulUrl
-		mConfig.Metadata[api.MachineConfigMetadataKeyFlyManagedPostgres] = "true"
-	}
-
-	mConfig.Mounts = nil
-	if len(mConfig.Mounts) > 0 {
-		switch {
-		case m.isPostgres:
-			mConfig.Mounts = nil
-		case len(alloc.AttachedVolumes.Nodes) > 1:
-			fmt.Fprintf(m.io.Out, "WARNING: alloc '%s' has more than one volume attached, using only the first\n", alloc.ID)
-			fallthrough
-		case len(alloc.AttachedVolumes.Nodes) > 0:
-			// FIXME: we should probably error out if there are more than 1 vols attached
-			vol := alloc.AttachedVolumes.Nodes[0]
-			mConfig.Mounts[0].Volume = vol.ID
-		default:
-			return nil, fmt.Errorf("application config has a [mounts] section but nomad alloc has no volume attached")
-		}
-	}
-
-	launchInput := &api.LaunchMachineInput{
-		AppID:   m.appFull.Name,
-		OrgSlug: m.appFull.Organization.ID,
-		Region:  alloc.Region,
-		Config:  mConfig,
-	}
-
-	return launchInput, nil
+func (m *v2PlatformMigrator) requiresDowntime() bool {
+	return m.usesForkedVolumes
 }
 
 func (m *v2PlatformMigrator) determinePrimaryRegion(ctx context.Context) error {
@@ -902,7 +712,7 @@ func (m *v2PlatformMigrator) determinePrimaryRegion(ctx context.Context) error {
 		m.appConfig.PrimaryRegion = fromFlag
 		return nil
 	}
-	fmt.Println(m.appConfig.Env)
+
 	if val, ok := m.appConfig.Env["PRIMARY_REGION"]; ok {
 		m.appConfig.PrimaryRegion = val
 		return nil
@@ -959,8 +769,16 @@ func (m *v2PlatformMigrator) ConfirmChanges(ctx context.Context) (bool, error) {
 		}
 	}
 
-	if !m.canAvoidDowntime {
+	if m.requiresDowntime() {
 		printAllocs("")
+		colorize := m.io.ColorScheme()
+		fmt.Fprintf(m.io.Out, "   * %s: Because your app uses volumes, there will be a short downtime during migration while your machines start up.\n", colorize.Bold(colorize.Yellow("NOTE")))
+	}
+	if m.usesForkedVolumes {
+		fmt.Fprintf(m.io.Out, " * Create clones of each volume in use, for the new machines\n")
+		fmt.Fprintf(m.io.Out, "   * These cloned volumes will have the suffix '%s' appended to their names\n", forkedVolSuffix)
+		fmt.Fprintf(m.io.Out, "   * Please note that your old volumes will not be removed.\n")
+		fmt.Fprintf(m.io.Out, "     (you can do this manually, after making sure the migration was a success)\n")
 	}
 
 	fmt.Fprintf(m.io.Out, " * Create machines, copying the configuration of each existing VM\n")
@@ -976,7 +794,7 @@ func (m *v2PlatformMigrator) ConfirmChanges(ctx context.Context) (bool, error) {
 		fmt.Fprintf(m.io.Out, " * Wait for at least one new in-region PG replica to become synced\n")
 	}
 
-	if m.canAvoidDowntime {
+	if !m.requiresDowntime() {
 		printAllocs("after health checks pass for the new machines")
 	}
 
@@ -1001,10 +819,7 @@ func (m *v2PlatformMigrator) ConfirmChanges(ctx context.Context) (bool, error) {
 }
 
 func determineAppConfigForMachines(ctx context.Context) (*appconfig.Config, error) {
-	var (
-		err                error
-		appNameFromContext = appconfig.NameFromContext(ctx)
-	)
+	appNameFromContext := appconfig.NameFromContext(ctx)
 	cfg, err := appconfig.FromRemoteApp(ctx, appNameFromContext)
 	if err != nil {
 		return nil, err
