@@ -110,7 +110,7 @@ ENV BUILT_BY_DOCKERFILE=true
 		f.Fatalf("failed to write dockerfile at %s error: %v", dockerfilePath, err)
 	}
 
-	f.Fly("deploy")
+	f.Fly("deploy --detach")
 }
 
 func TestAppsV2ConfigChanges(t *testing.T) {
@@ -121,7 +121,7 @@ func TestAppsV2ConfigChanges(t *testing.T) {
 		configFilePath = filepath.Join(f.WorkDir(), appconfig.DefaultConfigFileName)
 	)
 
-	f.Fly("launch --org %s --name %s --region %s --image nginx --force-machines --internal-port 7777 --now --auto-confirm", f.OrgSlug(), appName, f.PrimaryRegion())
+	f.Fly("launch --org %s --name %s --region %s --image nginx --force-machines --internal-port 80 --detach --now --auto-confirm", f.OrgSlug(), appName, f.PrimaryRegion())
 
 	f.Fly("config save -a %s -y", appName)
 	configFileBytes, err := os.ReadFile(configFilePath)
@@ -129,16 +129,16 @@ func TestAppsV2ConfigChanges(t *testing.T) {
 		f.Fatalf("error trying to read %s after running fly config save: %v", configFilePath, err)
 	}
 
-	newConfigFile := strings.Replace(string(configFileBytes), "internal_port = 7777", "internal_port = 9999", 1)
+	newConfigFile := strings.Replace(string(configFileBytes), `grace_period = "5s"`, `grace_period = "3s"`, 1)
 	err = os.WriteFile(configFilePath, []byte(newConfigFile), 0666)
 	if err != nil {
 		f.Fatalf("error trying to write to fly.toml: %s", err)
 	}
 
-	f.Fly("deploy --force-machines")
+	f.Fly("deploy --detach")
 
 	result := f.Fly("config show -a %s", appName)
-	require.Contains(f, result.StdOut().String(), `"internal_port": 9999`)
+	require.Contains(f, result.StdOut().String(), `"internal_port": 80`)
 
 	f.Fly("config save -a %s -y", appName)
 	configFileBytes, err = os.ReadFile(configFilePath)
@@ -146,7 +146,7 @@ func TestAppsV2ConfigChanges(t *testing.T) {
 		f.Fatalf("error trying to read %s after running fly config save: %v", configFilePath, err)
 	}
 
-	require.Contains(f, string(configFileBytes), "internal_port = 9999")
+	require.Contains(f, string(configFileBytes), `grace_period = "3s"`)
 }
 
 func TestAppsV2ConfigSave_ProcessGroups(t *testing.T) {
@@ -378,7 +378,7 @@ func TestAppsV2Config_ParseExperimental(t *testing.T) {
 		f.Fatalf("Failed to write config: %s", err)
 	}
 
-	result := f.Fly("launch --force-machines --name %s --region ord --copy-config --org %s", appName, f.OrgSlug())
+	result := f.Fly("launch --no-deploy --force-machines --name %s --region ord --copy-config --org %s", appName, f.OrgSlug())
 	stdout := result.StdOut().String()
 	require.Contains(f, stdout, "Created app")
 	require.Contains(f, stdout, "Wrote config file fly.toml")
@@ -414,7 +414,7 @@ func TestAppsV2Config_ProcessGroups(t *testing.T) {
 		if err != nil {
 			f.Fatalf("error trying to write %s: %v", configFilePath, err)
 		}
-		cmd := f.Fly("deploy --now --image nginx")
+		cmd := f.Fly("deploy --detach --now --image nginx")
 		cmd.AssertSuccessfulExit()
 		return cmd
 	}
@@ -573,7 +573,6 @@ web = "nginx -g 'daemon off;'"
 		}
 	}
 	require.Equal(t, idMatchFound, true, "could not find 'web' machine with matching machine ID")
-
 }
 
 func TestAppsV2MigrateToV2(t *testing.T) {
@@ -595,3 +594,54 @@ func TestAppsV2MigrateToV2(t *testing.T) {
 	platformVersion, _ := statusMap["PlatformVersion"].(string)
 	require.Equal(f, "machines", platformVersion)
 }
+
+// This test takes forever. I'm sorry.
+func TestAppsV2MigrateToV2_Volumes(t *testing.T) {
+	var (
+		err     error
+		f       = testlib.NewTestEnvFromEnv(t)
+		appName = f.CreateRandomAppName()
+	)
+	// No spaces or quotes, this is sent unescaped to bash :x
+	successStr := "myvolumehasloaded"
+
+	f.Fly("launch --org %s --name %s --region %s --internal-port 80 --force-nomad --image nginx", f.OrgSlug(), appName, f.PrimaryRegion())
+	f.Fly("vol create -y --app %s -s 2 --region %s vol_test", appName, f.PrimaryRegion())
+	{
+		toml, err := os.ReadFile("fly.toml")
+		if err != nil {
+			f.Fatalf("failed to read fly.toml: %s\n", err)
+		}
+		tomlStr := string(toml) + "\n[[mounts]]\n  source = \"vol_test\"\n  destination = \"/vol\"\n"
+		if err = os.WriteFile("fly.toml", []byte(tomlStr), 0644); err != nil {
+			f.Fatalf("failed to write fly.toml: %s\n", err)
+		}
+	}
+
+	assertHasFlag := func() {
+		output := f.Fly("ssh console -q -C 'cat /vol/flag.txt'")
+		output.AssertSuccessfulExit()
+		outStr := string(output.StdOut().Bytes())
+
+		require.Contains(t, outStr, successStr)
+	}
+
+	f.Fly("deploy --detach --now")
+	f.Fly("ssh console -C \"bash -c 'echo %s > /vol/flag.txt && sync'\"", successStr)
+
+	assertHasFlag()
+
+	time.Sleep(6 * time.Second)
+	f.Fly("migrate-to-v2 --primary-region %s --yes --experimental-volume-migration", f.PrimaryRegion())
+	result := f.Fly("status --json")
+
+	var statusMap map[string]any
+	err = json.Unmarshal(result.StdOut().Bytes(), &statusMap)
+	if err != nil {
+		f.Fatalf("failed to parse json: %v [output]: %s\n", err, result.StdOut().String())
+	}
+	platformVersion, _ := statusMap["PlatformVersion"].(string)
+	require.Equal(f, "machines", platformVersion)
+
+	assertHasFlag()
+}
